(* somme [x1; x2; ...; xn] = x1 + ... + xn *)
let rec somme (l: int list) : int =
	match l with 
	| [] 	 -> 0
	| x :: q -> x + somme q

(* renvoie true si l contient x, false sinon *)
let rec recherche (l: 'a list) (x: 'a) : bool =
	match l with 
	| [] -> false
	| y :: q -> x = y || recherche q x

(* renvoie le i-ème élément de l. Lève une erreur si
   la liste est de taille inférieure à i *)
let rec n_eme (l: 'a list) (i: int) : 'a =
	match (l, i) with 
	| [], _ -> failwith "Liste trop courte"
	| x :: q, 0 -> x
	| x :: q, _ -> n_eme q (i-1)

(* Renvoie la liste des éléments x de l tels que f x = true *)
let rec filter (f: 'a -> bool) (l: 'a list) : 'a list =
	match l with 
	| [] -> [] 
	| x :: q -> if f x then x :: filter f q
		        else filter f q

(* concatener [x1; ... xn] [y1; ...; ym] = [x1; ...; xn; y1; ... ym] *)
let rec concatener (l1: 'a list) (l2: 'a list) : 'a list =
	match l1 with 
	| []     -> l2
	| x :: q -> x :: concatener q l2


(* Renvoie la concaténation des listes de ll *)
let rec multi_concat (ll: 'a list list) : 'a list =
	match ll with 
	| [] -> []
	| l :: qq -> concatener l (multi_concat qq)

(* map f [x1; ...; xn] =  [f x1; ...; f xn] *)
let rec map (f: 'a -> 'b) (l: 'a list) : 'b list =
	match l with 
	| [] -> []
	| x :: q -> f x :: map f q