let rec taille (l: int list) : int =
	match l with 
	| [] 	 -> 0
	| x :: q -> 1 + taille q

let rec somme (l: int list) : int =
	match l with 
	| [] 	 -> 0
	| x :: q -> x + somme q

let rec recherche (l: 'a list) (x: 'a) : bool =
	match l with 
	| [] -> false
	| y :: q -> x = y || recherche q x

let rec filter (f: 'a -> bool) (l: 'a list) : 'a list =
	match l with 
	| [] -> [] 
	| x :: q -> if f x then x :: filter f q
		        else filter f q

let rec map (f: 'a -> 'b) (l: 'a list) : 'b list =
	match l with 
	| [] -> []
	| x :: q -> f x :: map f q

let rec string_cat (l: string list) : string =
	match l with 
	| [] 	 -> ""
	| x :: q -> x ^ string_cat q

(* fold f [x1; x2; ...; xn] b = f x1 (f x2 (.... (f xn b)...)) *)
let rec fold (f: 'a -> 'b -> 'b) (l: 'a list) (b: 'b) =
	match l with 
	| [] -> b
	| x :: q -> f x (fold f q b)

let taille2 (l: 'a list) : int =
	(* si on a déjà une taille t, et qu'on croise un élément x,
	   la nouvelle taille est t+1. La taille d'une liste vide est 0 *)
	fold (fun x t -> t + 1) l 0

let recherche2 (l: 'a list) (x: 'a) : bool =
	(* si b est un booléen indiquant si x a été trouvé,
	   et qu'on croise un élément y, alors la nouvelle
	   valeur de vérité de recherche est: x=y || b *)
	fold (fun y b -> x = y || b ) l false

let filter2 (f: 'a -> bool) (l: 'a list) : 'a list =
	fold (fun x l' -> if f x then x :: l' else l') l []

let map2 (f: 'a -> 'b) (l: 'a list) : 'b list =
	fold (fun x l' -> f x :: l') l []

(*** Fonction range ***)
(* Pour faire cette fonction, il faut passer par
   une fonction auxiliaire. Une possibilité est
   de rajouter un paramètre i permettant de faire
   une sorte de récurrence montante:  *)

(* range_from i n = [i; i+1; ...; n-1] *)
let rec range_from (i: int) (n: int) : int list =
	if i >= n then [] 
	else i :: range_from (i+1) n

(* range n = [0; 1; ...; n-1] *)
let range n = range_from 0 n


(* renvoie la somme des carrés des diviseurs de n *)
let somme_carres_div (n: int) =
	(* liste des entiers de 1 à n *)
	let diviseurs_potentiels = map (fun x -> x + 1) (range n) in
	(* liste des diviseurs de n *)
	let diviseurs = filter (fun d -> n mod d = 0) diviseurs_potentiels in
	let carres = map (fun x -> x*x) diviseurs in
	fold (+) carres 0

let test () =
	assert (somme_carres_div 6 = 50); (* 1 + 4 + 9 + 36 *)
	assert (somme_carres_div 11 = 122);
	assert (somme_carres_div 1 = 1)