(* Renvoie le minimum de l. l doit être non-vide *)
let rec find_min (l: 'a list) : 'a =
	match l with 
	| []      -> failwith "Liste vide"
	| x :: [] -> x
	| x :: q  -> let m = find_min q in 
		         if m < x then m else x

		         
(* Renvoie un couple (m, l') avec:
   - m minimum de l. 
   - l' = l privée d'une occurrence de m
   Hypothèse: l doit être non-vide *)
let rec extract_min (l: 'a list) : ('a * 'a list) =
	match l with 
	| []      -> failwith "Liste vide"
	| x :: [] -> (x, [])
	| x :: q  -> let (m, q') = extract_min q in 
		         if m < x then (m, x::q') else (x, q)

(* Renvoie une copie triée de l *)
let rec select_sort (l: 'a list) : 'a list =
	match l with 
	| [] -> []
	| _ -> let (m, q) = extract_min l in
		   m :: select_sort q

let test_select_sort () =
	assert (select_sort [3;1;4;1;5] = [1;1;3;4;5]);
	assert (select_sort [7;6;5;4;3;2;1] = [1;2;3;4;5;6;7])

(* Insère x dans l. Si l est triée, alors la liste renvoyée aussi *)
let rec insert (x: 'a) (l: 'a list) : 'a list =
	match l with 
	| [] -> [x]
	| y ::q -> if x < y then x :: y :: q
		       else y :: insert x q

let rec insert_sort (l :'a list) : 'a list =
	match l with 
	| [] -> []
	| x ::q -> let q_triee = insert_sort q in
		       insert x q_triee

let test_insert_sort () =
	assert (insert_sort [3;1;4;1;5] = [1;1;3;4;5]);
	assert (insert_sort [7;6;5;4;3;2;1] = [1;2;3;4;5;6;7])


(** Tri fusion **)

(* Renvoie une liste triée contenant les éléments de l1 et l2
   Hypothèse: l1 et l2 sont triées *)
let rec fusion (l1: 'a list) (l2: 'a list) : 'a list =
	match l1, l2 with 
	| [], _ -> l2
	| _, [] -> l1
	| x1 :: q1, x2 :: q2 ->
		if x1 < x2 then x1 :: fusion q1 l2
		else            x2 :: fusion l1 q2

(* renvoie deux listes de tailles égales (à 1 près) contenant
   les éléments de l *)
let rec separer (l: 'a list) : 'a list * 'a list =
	match l with 
	| [] -> ([], [])
	| [x] -> ([x], [])
	| x :: y :: q ->
		(* mettre x dans une liste, y dans l'autre *)
		let l1, l2 = separer q in
		(x :: l1, y:: l2)


let rec tri_fusion (l :'a list) : 'a list =
	match l with 
	| [] -> []
	| [x] -> [x]
	| _ -> let l1, l2 = separer l in
		   let l1_triee = tri_fusion l1 in
		   let l2_triee = tri_fusion l2 in
		   fusion l1_triee l2_triee

let test_tri_fusion () =
	assert (tri_fusion [3;1;4;1;5] = [1;1;3;4;5]);
	assert (tri_fusion [7;6;5;4;3;2;1] = [1;2;3;4;5;6;7])


let test () =
	test_select_sort ();
	test_insert_sort ();
	test_tri_fusion ();
	print_string "Tous les tests ont réussi"

(* taper test ();; dans utop pour lancer les trois tests *)